#######################################################################
# Copyright (c) 2018 Josef Spidlen, Ph.D., FlowJo, LLC
#
# License
# The software is distributed under the terms of the 
# Artistic License 2.0
# http://www.r-project.org/Licenses/Artistic-2.0
# 
# Disclaimer
# This software and documentation come with no warranties of any kind.
# This software is provided "as is" and any express or implied 
# warranties, including, but not limited to, the implied warranties of
# merchantability and fitness for a particular purpose are disclaimed.
# In no event shall the  copyright holder be liable for any direct, 
# indirect, incidental, special, exemplary, or consequential damages
# (including but not limited to, procurement of substitute goods or 
# services; loss of use, data or profits; or business interruption)
# however caused and on any theory of liability, whether in contract,
# strict liability, or tort arising in any way out of the use of this 
# software.
######################################################################
  writeLines("Entered R")

if(!require('ggplot2'))
{
  writeLines(" ggplot2 is not installed, installing...")
 
  install.packages("ggplot2")

  library("ggplot2")
}else{
  writeLines(" Start loading ggplot2")
  library(ggplot2)
}
#if(!require('RColorBrewer'))
#{
 # writeLines(" RColorBrewer is not installed, installing...")
 #
  #install.packages("RColorBrewer")
#
 # library("RColorBrewer")
#}else{
 # writeLines(" Start loading RColorBrewer")
  #library(RColorBrewer)
#}
sessionInfo()
  writeLines("Loaded packages")
setwd("FJ_OUTPUT_DIR")
outPath <- "FJ_CSV_OUPUT_FILE"
inCSVPath <- "FJ_DATA_FILE_PATH"
sampleName <- "FJ_SAMPLE_NAME"
applyOn <- "FJ_APPLY_ON"

parFlipGraph <- FJ_FLIP_GRAPH
parTrim <- FJ_TRIM
parJitter <- FJ_JITTER
parCreatePDF <- FJ_PDF
parRemoveBg <- FJ_REMOVE_BG

parHeight <- FJ_HEIGHT
parWidthPerFig <- FJ_WIDTH
parExtraWidthForLegend <- FJ_WIDTH2
parLegendSize <- FJ_LEGEND_SIZE ## Base (100%) is 24 point
parJitterSize <- FJ_JIT_SIZE    ## Base (100%) is 0.5
parJitterDisp <- FJ_JIT_DISP
parJitterDodge <- FJ_JIT_DODGE

parMaxSampleNameLen <- FJ_MAX_SAMPLE_NAME_LEN

parLegendPosition <- "FJ_LEGEND_POS" ## "left", "top", "right", "bottom", "none".

legendSize <- as.integer((parLegendSize * 24) / 100)
jitterSize <- (parJitterSize * 0.5 / 100)
jitterDisp <- parJitterDisp / 100
jitterDodge <- parJitterDodge / 100

if (parLegendPosition != "left" && parLegendPosition != "right") {
	## No extra width for legend if legend unless legend is on the left or right 
	parExtraWidthForLegend  <- 0
}

robustLog2 <- function(x) {
	x <- as.numeric(x)
	if (x >= 0) {
		log(x + 1, 2)
	} else {
		0
	}
}

substrRight <- function(x, n){
	substr(x, nchar(x)-n+1, nchar(x))
}

if (nchar(sampleName) > parMaxSampleNameLen) {
	sampleName <- paste0("...", substrRight(sampleName, parMaxSampleNameLen-3))
}
  writeLines("Start loading CSV ")

input <- (read.csv(inCSVPath, header = T, sep = ",", fill = F, na.strings = " " ))
  writeLines("Finished loading CSV ")

if(APP_FLOWJO){
  writeLines("Is FlowJo")

 whichColDP <- which(colnames(input)=="EventNumberDP")
  if(length(whichColDP)>0){
    input = input[,-whichColDP]
  }
}

#colnames(input) <- input[1,]
#if (dim(input)[2] > 1) {
#	input <- input[-1,]
#} else {
	#tmpNames <- input[1,]
	#input <- as.matrix(input[-1,])
	#colnames(input) <- tmpNames
#}
  writeLines("Cleaned CSV")

#colnames(input)
cellNum <- dim(input)[1]
geneNum <- dim(input)[2]
  writeLines("Created Dataframe")

inputDF <- data.frame(Log2Expression=numeric(), Parameter =character(), Sample=character(), stringsAsFactors=FALSE) 
if (applyOn != "NONE") {
	load(applyOn)
	inputDF$Parameter  <- as.character(inputDF$Parameter )
	inputDF$Sample <- as.character(inputDF$Sample)
}
	
for(g in 1:geneNum) {
	for(c in 1:cellNum) {
		if (dim(inputDF)[1] > 0)
			inputDF <- rbind(inputDF, c(robustLog2(input[c,g]), c(colnames(input)[g]), sampleName))
		else
			inputDF <- data.frame(Log2Expression=robustLog2(input[c,g]), Parameter =colnames(input)[g], Sample=sampleName, stringsAsFactors=FALSE)
	}
}
  writeLines("Populated Dataframe")

inputDF$Log2Expression <- as.numeric(inputDF$Log2Expression)
#inputDF$Parameter  <- as.factor(inputDF$Parameter )
inputDF$Sample <- as.factor(inputDF$Sample)

## Other options that may be explored?

if (applyOn != "NONE") {
    if (FJ_BnW) {
	p <- ggplot(inputDF, aes(x=Parameter , y=Log2Expression, fill=Sample)) + geom_boxplot()
	}	
    else { 
        p <- ggplot(inputDF, aes(x=Parameter , y=Log2Expression, fill=Sample)) + geom_violin(trim=parTrim)}
	if (parJitter) {
		## Other options that may be explored?
		#p <- p + geom_jitter(shape=16, position=position_jitter(0.1), aes(colour = Sample))
		p <- p + geom_jitter(shape=16, size=jitterSize, position=position_jitterdodge(jitter.height = jitterDisp, dodge.width = jitterDodge))
	}
} else {
	if (FJ_BnW) {
	p <- ggplot(inputDF, aes(x=Parameter , y=Log2Expression, fill=Parameter )) + geom_boxplot()
	}	
    else { 
        p <- ggplot(inputDF, aes(x=Parameter , y=Log2Expression, fill=Parameter )) + geom_violin(trim=parTrim)}
	if (parJitter) {
		p <- p + geom_jitter(shape=16, size=jitterSize, position=position_jitter(jitterDisp))
	}
}
  writeLines("Created Plot")

## Other options that may be explored?
# p + stat_summary(fun.data=mean_sdl, geom="pointrange", color="red")
# p + geom_boxplot(width=0.1)
# p + stat_summary(fun.y=median, geom="point", size=2, color="red")
# p + stat_summary(fun.y=mean, geom="point", shape=23, size=2)
# p + geom_jitter(shape=16, position=position_jitter(0.1), color="red")
# p + geom_jitter(shape=16, position=position_jitter(0.1))

p <- p + theme_gray(base_size=legendSize)
p <- p + theme(legend.position=parLegendPosition)
#p <- p + scale_fill_brewer(palette="Pastel2")


if (parRemoveBg) {
	p <- p + theme(
		panel.background = element_rect(fill = "transparent"), # bg of the panel
		plot.background = element_rect(fill = "transparent"), # bg of the plot
		# panel.grid.major = element_blank(), # get rid of major grid
		# panel.grid.minor = element_blank(), # get rid of minor grid
		legend.background = element_rect(fill = "transparent"), # get rid of legend bg
		legend.box.background = element_rect(fill = "transparent") # get rid of legend panel bg
	)
}

if (parFlipGraph) { 
	p <- p + coord_flip()
}

numSamples <- length(unique(inputDF$Sample))
  writeLines("Write PNG")

png(file=paste0(outPath, ".png"), bg="transparent", width = parExtraWidthForLegend + parWidthPerFig * geneNum * numSamples, height = parHeight, units = "px", pointsize = legendSize)
plot(p)
dev.off()
  writeLines("Finished PNG")

if(parCreatePDF)
{
	ggsave(paste0(outPath, ".pdf"))
}
  writeLines("Created PDF")

save(inputDF, file=paste0(outPath, ".viDF.RData"))
  writeLines("Saved file")

# R seems to be saving the .RData when exiting, so let's clean up to at least make that tiny (i.e., empty environment)
rm(list=ls())
writeLines("Cleaned R")
